import { Component, OnInit, Input, Output, OnDestroy, EventEmitter } from '@angular/core';
import { Exercise, IExerciseType, searchResultToExercise } from 'app/shared/model/exercise.model';
import { ExerciseService } from 'app/exercise/service/exercise.service';
import { Subscription } from 'rxjs';
import { PluginActionInfo } from 'app/shared/model/search/search-result-dto.model';
import { PluginService } from 'app/shared/service/plugin-service';
import { ShoppingBasketInfo, ShoppingBasketRedirectInfoDTO } from 'app/shared/model/basket/shopping-basket-info.model';
import { AccountService } from 'app/core/auth/account.service';
import { Account } from 'app/core/user/account.model';
import { SearchService } from 'app/search/service/search-service';
import { HttpResponse } from '@angular/common/http';
import { JhiAlertService } from 'ng-jhipster';
import { WatchlistManager } from 'app/shared/watchlist/watchlist-manager';
import { LikesService } from 'app/entities/likes/likes.service';
import { faArchive } from '@fortawesome/free-solid-svg-icons';

import { Router } from '@angular/router';

@Component({
  selector: 'jhi-exercise-details',
  templateUrl: './exercise-details.component.html',
  styleUrls: ['./exercise-details.component.scss'],
})
export class ExerciseDetailsComponent implements OnInit, OnDestroy {
  @Input() exercise: Exercise | undefined;
  @Output() exerciseChangedEvent = new EventEmitter<Exercise>();

  account: Account | null = null;
  authSubscription?: Subscription;
  hasLiked: Boolean | null = null;
  likeSubscription?: Subscription;
  authenticated = false;
  treeIcon = faArchive;

  constructor(
    private accountService: AccountService,
    protected pluginService: PluginService,
    private searchService: SearchService,
    private likesService: LikesService,
    private jhiAlertService: JhiAlertService,
    private watchlistManager: WatchlistManager,
    private exerciseService: ExerciseService,
    private router: Router
  ) {}

  ngOnInit(): void {
    this.authSubscription = this.accountService.getAuthenticationState().subscribe(account => (this.account = account));
  }

  public get exerciseType(): typeof IExerciseType {
    return IExerciseType;
  }

  public isAuthenticated(): boolean {
    return this.accountService.isAuthenticated();
  }

  isOnCurrentWatchlist(e: Exercise): boolean {
    return this.watchlistManager.isExerciseOnCurrentWatchlist(e);
  }

  handleForCurrentWatchlist(e: Exercise) {
    this.watchlistManager.handleCheckForCurrentWatchlist(e);
  }

  public startAction(action: PluginActionInfo, exercise: Exercise): void {
    const basketInfo: ShoppingBasketInfo = {
      plugin: action.plugin,
      action: action.action,
      itemInfos: [exercise.originalResult],
    };
    this.pluginService.getRedirectLink(basketInfo).subscribe(
      (redirectInfo: ShoppingBasketRedirectInfoDTO) => {
        // alert('redirecting to ' + redirectInfo.redirectURL);
        // location.href = redirectInfo.redirectURL;
        window.open(redirectInfo.redirectURL, action.action);
      },
      () => alert('Search failed')
    );
  }

  public download(): void {
    this.exportProject(Number(this.exercise!.originalResult.project.project_id));
  }

  exportProject(projectId: number) {
    return this.searchService.exportProject(projectId).subscribe(
      (response: HttpResponse<Blob>) => {
        this.jhiAlertService.success('artemisApp.programmingExercise.export.successMessage');
        if (response.body) {
          const zipFile = new Blob([response.body], { type: 'application/zip' });
          const url = window.URL.createObjectURL(zipFile);
          const link = document.createElement('a');
          link.setAttribute('href', url);
          link.setAttribute('download', response.headers.get('filename')!);
          document.body.appendChild(link); // Required for FF
          link.click();
          window.URL.revokeObjectURL(url);
        }
      },
      () => alert('Unable to export exercise. Please log in to export, or check your git lab permissions.')
    );
  }

  openLink(link: string): void {
    window.open(link);
  }

  searchChildren(parentId: string): void {
    this.router.navigate(['/search'], { queryParams: { p: parentId } });
  }

  toParent(parentId: string): void {
    this.exerciseService.loadExercise(parentId).subscribe(searchResult => {
      this.exercise = searchResultToExercise(searchResult);
      this.exerciseChangedEvent.emit(this.exercise);
    }),
      () => {
        alert('parent cannot be loaded');
      };
  }

  ngOnDestroy(): void {
    if (this.authSubscription) {
      this.authSubscription.unsubscribe();
    }
  }

  /**
   * correct missing image urls
   */
  correctImageURL(event: Event): void {
    if (event.srcElement) {
      event.srcElement['src'] = '/content/images/Logo_codeAbility_4c_300dpi_RGB3.gif';
    }
  }

  likeAction(): void {
    // to do call like service
    this.likesService.likeExercise(this.exercise!.originalResult.project.project_id).subscribe(() =>
      // eslint-disable-next-line no-console
      console.log('Liked post ' + this.exercise!.originalResult.project.project_id)
    );

    this.exercise!.numberOfLikes = this.exercise!.numberOfLikes + 1;
    this.exercise!.userHasLiked = true;

    // eslint-disable-next-line no-console
    console.log('Reloaded number of likes is ' + this.exercise!.numberOfLikes);
  }

  unlikeAction(): void {
    // to do call like service
    this.likesService.unlikeExercise(this.exercise!.originalResult.project.project_id).subscribe(() =>
      // eslint-disable-next-line no-console
      console.log('Unliked post ' + this.exercise!.originalResult.project.project_id)
    );

    this.exercise!.numberOfLikes = this.exercise!.numberOfLikes - 1;
    this.exercise!.userHasLiked = false;

    // eslint-disable-next-line no-console
    console.log('Reloaded number of likes is ' + this.exercise!.numberOfLikes);
  }

  selectREADME(): void {}
}
